use crate::scene::*;
use crate::offaxis::*;
use crate::screeninfo;
use crate::screeninfo::ScreenInfo;
use crate::viewer::Viewer;


use bevy::{prelude::*, window::PresentMode, render::camera::CameraProjectionPlugin};

pub fn cycle_msaa(input: Res<Input<KeyCode>>, mut msaa: ResMut<Msaa>) {
    if input.just_pressed(KeyCode::M) {
        if msaa.samples == 4 {
            info!("Not using MSAA");
            msaa.samples = 1;
        } else {
            info!("Using 4x MSAA");
            msaa.samples = 4;
        }
    }
}

// pub fn update_offaxis(mut query : Query<(
//     &mut OffAxisProjection,
//     &mut Transform,
//     &ScreenInfo,
//     &Viewer
// )>
// ) {
//     for (mut projection,mut transform,screen_info,viewer) in query.iter_mut() {

//         // we fake access to far for updating the matrix
//         (*projection).far *= 1.0;

//         info!("Projection {:?}",*projection);
        
//         info!("Screen Info {:?}",screen_info.name);

//         info!("Viewer {:?}",viewer.position);

//         *transform = Transform::from_translation(viewer.position).with_rotation(transform.rotation);



//     }
// }


fn intersection_point( 
    center_point: Vec3, 
    normal: Vec3,
    p1: Vec3, 
    p2: Vec3) -> (bool, Vec3) {
    let u = normal.dot(center_point - p1) / normal.dot(p2 - p1);

    let intersection_point = p1.lerp(p2,u); //p1 + ((p2 - p1) * u);

    // returns a tuple if the intersection point is in front and the actual point
    (u >= 0.0 && u <= 1.0, intersection_point)
}

fn normal_intersection_point(
    center_point: Vec3,
    normal : Vec3,
    p1: Vec3) -> (bool, Vec3) 
{
    let p2 = p1 - normal * 10.0;

    intersection_point(center_point, normal, p1, p2)
}